local M = {}

local helper = require("scenario/scenariohelper")
local player = nil
local playerPos = nil
local playerDistance = nil
local racers = {}
local racerPos = nil
local racerPosMark = nil
local racerState = {}
local markers = {}
local markerTimer = 0
local markerTimerMax = 0
local arrestTimer = {}
local score = 0
local delay = 0
local totalRacers = 0
	
local function winScenario()
	score = score + 1;
	if score >= totalRacers then
		scenetree.findObject("cta_win"):setPosition(vec3(10000,10000,10000):toPoint3F())
	end
end

local function updateMarker(markerI, racerPosMark)
	markerI.hidden = false
	markerI:setPosition(racerPosMark:toPoint3F())
	markerI:setScale(Point3F(5, 5, 60))
	markerI.instanceColor = ColorF(1, 0, 0, 1):asLinear4F()
end

local function arrestRacer(racerI,id)
	local racerVel = map.objects[map.objectNames[racerI.name]].vel:length()
	if racerVel < 3 and racerState[id] then
		if arrestTimer[id] % 1 == 0 then
			local countdown = 5 - math.modf(arrestTimer[id])
			if countdown > 0 then
				helper.flashUiMessage((tostring(countdown).."..."), 0.5, true)
			end
		end
		arrestTimer[id] = arrestTimer[id] + 0.25
	else 
		arrestTimer[id] = 0
	end 	
	if arrestTimer[id] > 5 then
		helper.flashUiMessage("Busted!", 2, true)
		helper.setAiMode(racerI.name, "stopping")
		arrestTimer[id] = 0
		winScenario()
		racerState[id] = false
	end
end

local function onScenarioUIReady(state)
	core_environment.setTimeOfDay({time=0.16})
	markerTimer = 0
	score = 0
	delay = 0
	totalRacers = 0
	player = scenetree.findObject("cta_player0")
	racers = {scenetree.findObject("cta_ai0"), scenetree.findObject("cta_ai1"), scenetree.findObject("cta_ai2")}
	markers = {scenetree.findObject("cta_marker0"), scenetree.findObject("cta_marker1"), scenetree.findObject("cta_marker2")}
	for i, v in ipairs(racers) do
		racerState[i] = true
		arrestTimer[i] = 0
		totalRacers = totalRacers + 1
		helper.setAiMode(racers[i].name, "flee")
	end
end

local function onRaceTick(raceTickTime)
	local markerState = 0
	for i, v in ipairs(racers) do
		playerPos = player:getPosition()
		racerPos = racers[i]:getPosition()
		playerDistance = (playerPos - racerPos):len()
		if racerState[i] and playerDistance >= 100 then
			markerState = 15
		else
			markerState = 2000
			if delay >= 20 and playerDistance <= 50 then
				arrestRacer(racers[i],i)
			else
				delay = delay + 1
			end
		end
		if markerTimer >= markerTimerMax then
			racerPosMark = vec3(racerPos.x, racerPos.y, racerPos.z + markerState)
			updateMarker(markers[i], racerPosMark)
		end	
	end
	if markerTimer >= markerTimerMax then
		markerTimer = 0
	end
	markerTimer = markerTimer + 1
end

M.onScenarioUIReady = onScenarioUIReady
M.onRaceTick = onRaceTick
M.winScenario = winScenario
M.updateMarker = updateMarker
M.arrestRacer = arrestRacer
return M